
/**
   @version 1.00 2002-07-10
   @author Cay Horstmann
*/

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Array;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.StringTokenizer;

import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultFormatter;
import javax.swing.text.DocumentFilter;
import javax.swing.text.InternationalFormatter;
import javax.swing.text.MaskFormatter;

/**
   A program for testing formatted text field. 
*/
public class JFormattedFieldTest
{
   public static void main(String[] args)
   {
      FormatTestFrame frame = new FormatTestFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setVisible(true);
   }
}

/**
   JFrame contains a collection of different FormattedTextFields. 
*/
class FormatTestFrame extends JFrame
{
   public FormatTestFrame()
   {
      setTitle("FormatTest");
      setSize(WIDTH, HEIGHT);

      Container contentPane = getContentPane();

      JPanel buttonPanel = new JPanel();
      okButton = new JButton("Ok");
      buttonPanel.add(okButton);
      contentPane.add(buttonPanel, BorderLayout.SOUTH);

      mainPanel = new JPanel();
      mainPanel.setLayout(new GridLayout(0, 3));
      contentPane.add(mainPanel, BorderLayout.CENTER);

      JFormattedTextField intField = new JFormattedTextField(
         NumberFormat.getIntegerInstance());
      intField.setValue(new Integer(100));
      addRow("Zahl:", intField);

      JFormattedTextField intField2 = new JFormattedTextField(
         NumberFormat.getIntegerInstance());
      intField2.setValue(new Integer(100));
      intField2.setFocusLostBehavior(JFormattedTextField.COMMIT);
      addRow("Zahl (Besttigen-Verhalten):", intField2);

      JFormattedTextField intField3
         = new JFormattedTextField(new
            InternationalFormatter(
               NumberFormat.getIntegerInstance())
            {
               protected DocumentFilter getDocumentFilter()
               {
                  return filter;
               }
               private DocumentFilter filter
                  = new IntFilter();
            });
      intField3.setValue(new Integer(100));
      addRow("Gefilterte Zahl:", intField3);

      JFormattedTextField intField4 = new JFormattedTextField(
         NumberFormat.getIntegerInstance());
      intField4.setValue(new Integer(100));
      intField4.setInputVerifier(
         new FormattedTextFieldVerifier());
      addRow("Verifizierte Zahl:", intField4);

      JFormattedTextField currencyField = new
         JFormattedTextField(NumberFormat.getCurrencyInstance());
      currencyField.setValue(new Double(10));
      addRow("Whrung:", currencyField);

      JFormattedTextField dateField = new JFormattedTextField(
         DateFormat.getDateInstance());
      dateField.setValue(new Date());
      addRow("Datum (Standard):", dateField);

      DateFormat format =
         DateFormat.getDateInstance(DateFormat.SHORT);
      format.setLenient(false);
      JFormattedTextField dateField2 = new JFormattedTextField(
         format);
      dateField2.setValue(new Date());
      addRow("Datum (kurz, nicht nachsichtig):", dateField2);

      try
      {
         DefaultFormatter formatter = new DefaultFormatter();
         formatter.setOverwriteMode(false);
         JFormattedTextField urlField = new JFormattedTextField(
            formatter);
         urlField.setValue(new URL("http://java.sun.com"));
         addRow("URL:", urlField);
      }
      catch (MalformedURLException exception)
      {
         exception.printStackTrace();
      }

      try
      {
         MaskFormatter formatter
            = new MaskFormatter("###-##-####");
         formatter.setPlaceholderCharacter('0');
         JFormattedTextField ssnField
            = new JFormattedTextField(formatter);
         ssnField.setValue("078-05-1120");
         addRow("SSN-Maske:", ssnField);
      }
      catch (ParseException exception)
      {
         exception.printStackTrace();
      }

      JFormattedTextField ipField
         = new JFormattedTextField(new IPAddressFormatter());
      ipField.setValue(new byte[] { (byte)130, 65, 86, 66 });
      addRow("IP-Adresse:", ipField);
   }

   /**
      Adds a row into the content pane. 
      @param labelText label for the text field
      @param field sample text field 
   */
   public void addRow(String labelText,
      final JFormattedTextField field)
   {
      mainPanel.add(new JLabel(labelText));
      mainPanel.add(field);
      final JLabel valueLabel = new JLabel();
      mainPanel.add(valueLabel);
      okButton.addActionListener(new
         ActionListener()
         {
            public void actionPerformed(ActionEvent event)
            {
               Object value = field.getValue();
               if (value.getClass().isArray())
               {
                  StringBuffer buffer = new StringBuffer();
                  buffer.append('{');
                  for (int i = 0; i < Array.getLength(value);
                       i++)
                  {
                     if (i > 0) buffer.append(',');
                     buffer.append(
                        Array.get(value, i).toString());
                  }
                  buffer.append('}');
                  valueLabel.setText(buffer.toString());
               }
               else
                  valueLabel.setText(value.toString());
            }
         });
   }

   public static final int WIDTH = 500;
   public static final int HEIGHT = 250;

   private JButton okButton;
   private JPanel mainPanel;
}

/**
   Filter for input of digits and minus sign. 
*/
class IntFilter extends DocumentFilter
{
   public void insertString(FilterBypass fb, int offset,
      String string, AttributeSet attr)
      throws BadLocationException
   {
      StringBuffer buffer = new StringBuffer(string);
      for (int i = buffer.length() - 1; i >= 0; i--)
      {
         char ch = buffer.charAt(i);
         if (!Character.isDigit(ch) && ch != '-')
            buffer.deleteCharAt(i);
      }
      super.insertString(fb, offset, buffer.toString(), attr);
   }

   public void replace(FilterBypass fb, int offset,
      int length, String string, AttributeSet attr)
      throws BadLocationException
   {
      if (string != null)
      {
         StringBuffer buffer = new StringBuffer(string);
         for (int i = buffer.length() - 1; i >= 0; i--)
         {
            char ch = buffer.charAt(i);
            if (!Character.isDigit(ch) && ch != '-')
               buffer.deleteCharAt(i);
         }
         string = buffer.toString();
      }
      super.replace(fb, offset, length, string, attr);
   }
}

/**
 * Verifier to check if content of text field is valid. 
*/
class FormattedTextFieldVerifier extends InputVerifier
{
   public boolean verify(JComponent component)
   {
      JFormattedTextField field = (JFormattedTextField)component;
      return field.isEditValid();
   }
}

/** A formatter for a 4 byte IP address in the form of a.b.c.d
*/
class IPAddressFormatter extends DefaultFormatter
{
   public String valueToString(Object value)
      throws ParseException
   {
      if (!(value instanceof byte[]))
         throw new ParseException("No byte[]", 0);
      byte[] a = (byte[])value;
      if (a.length != 4)
         throw new ParseException("Length != 4", 0);
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < 4; i++)
      {
         int b = a[i];
         if (b < 0) b += 256;
         buffer.append(String.valueOf(b));
         if (i < 3) buffer.append('.');
      }
      return buffer.toString();
   }

   /**
    * Conversion from string to value for IP address
    */
   public Object stringToValue(String text) throws ParseException
   {
      StringTokenizer tokenizer = new StringTokenizer(text, ".");
      byte[] a = new byte[4];
      for (int i = 0; i < 4; i++)
      {
         int b = 0;
         try
         {
            b = Integer.parseInt(tokenizer.nextToken());
         }
         catch (NumberFormatException e)
         {
            throw new ParseException("No integer", 0);
         }
         if (b < 0 || b >= 256)
            throw new ParseException("number out for range", 0);
         a[i] = (byte)b;
      }
      return a;
   }
}
